PROJECT_NAME = "FrameX"
VERSION = "0.2.3"
API_STR = "/api/v1"

BACKEND_NAME = "backend"
APP_NAME = "default"
PROXY_PLUGIN_NAME = "proxy.ProxyPlugin"

DEFAULT_ENV = {"RAY_COLOR_PREFIX": "1", "RAY_DEDUP_LOGS": "1", "RAY_SERVE_RUN_SYNC_IN_THREADPOOL": "1"}
